
package SimpleNumerics;

import ij.*;
import ij.process.*;
import ij.gui.*;
import java.awt.*;
import ij.plugin.*;

public class SimpleMatrix {
    public int rows;
    public int columns;
    public double tolerance = 1e-10;
    
    double[] mtx;
    
    public SimpleMatrix( int r, int c )
    {
	mtx = new double [ r * c ];
	rows = r;
	columns = c;
    }
    
    public double getElement(int r, int c)
    {
	return mtx [ r * columns + c ];
    }
    
    public void setElement(int r, int c, double value)
    {
	mtx [ r * columns + c ] = value;
    }
    
    public void show()
    {
	String str = new String();
	
	for( int i = 0; i < rows; i++ )
	    {
		for( int j = 0; j < columns; j++ )
		    {
			String s = String.format( "%.4f ", getElement( i, j ) );
			str += s;
		    }
		
		str += "\n";
	    }
	
	IJ.showMessage( str );
    }

    public SimpleMatrix rightMultiply( SimpleMatrix m2 )
    {
	int i, j, k;
	double sum;
	
	if( this.columns != m2.rows )
	    return null;

	SimpleMatrix out = new SimpleMatrix( this.rows, m2.columns );

	for( i = 0; i < this.rows; i ++ )
	    for( j = 0; j < m2.columns; j ++ ) {
		sum = 0.0;
		for( k = 0; k < this.columns; k++ )
		    {
			sum += this.getElement( i, k ) * m2.getElement( k, j );
		    }
		out.setElement( i, j, sum );
	    }
	
	return out;	
    }
    
    public int Jordan()
    {
	int i, j, k;
	double x;
	double value;
	
	for( i = 0; i < this.rows; i++ ) {
	    x = this.getElement( i, i );

	    if( Math.abs( x ) < tolerance ) {
		for( j = i + 1; j < this.rows; j++ )
		    if( Math.abs( this.getElement( i, i ) ) > tolerance )
			break;
		
		if ( j == this.rows )
		    return 0;
		
		for( k = 0; k < this.columns; k++ ) {
		    x = this.getElement( i, k );
		    value = this.getElement( j, k );
		    this.setElement( i, k, value );
		    this.setElement( j, k, x );
		}
		
		x = this.getElement(i, i);
	    }
		
	    // IJ.showMessage( "Foelem " + x );
		
	    this.setElement( i, i, 1.0 );
	    for( j = i + 1; j < this.columns; j++) {
		value = this.getElement( i, j ) / x;
		this.setElement( i, j, value );
		// IJ.showMessage( "Osztas utan "+ value );
	    }
		
	    for( j = 0; j < this.rows; j++ ) {
		if( j != i ) {
		    x = this.getElement( j, i );
		    this.setElement( j, i, 0.0 );
		    for( k = i + 1; k < this.columns; k++) {
			value = this.getElement( j, k ) - x * this.getElement( i, k );
			this.setElement( j, k, value );
		    }
		}
	    }
	}
	
	return 1;
    }
}
